#!/usr/bin/env python
"""
Some constants used by the samplers

"""

FILE_SUFFIX = ".out"

LOG_FILE_SUFFIX = ".log"

PROB_PREFIX = "prob"

BURNIN_PREFIX = "burnin"

STATE_PREFIX = "state"

PROB_SUFFIX = PROB_PREFIX+FILE_SUFFIX

BURNIN_SUFFIX = BURNIN_PREFIX+FILE_SUFFIX

BURNIN_PROB_SUFFIX = BURNIN_PREFIX+PROB_PREFIX+FILE_SUFFIX

BURNIN_STATE_SUFFIX = BURNIN_PREFIX+STATE_PREFIX+FILE_SUFFIX
